import { CancellationToken } from '../../../base/common/cancellation.js';
import { TunnelPrivacy } from '../../remote/common/remoteAuthorityResolver.js';

interface RemoteTunnel {
    readonly tunnelRemotePort: number;
    readonly tunnelRemoteHost: string;
    readonly tunnelLocalPort?: number;
    readonly localAddress: string;
    readonly privacy: string;
    readonly protocol?: string;
    dispose(silent?: boolean): Promise<void>;
}
interface TunnelOptions {
    remoteAddress: {
        port: number;
        host: string;
    };
    localAddressPort?: number;
    label?: string;
    public?: boolean;
    privacy?: string;
    protocol?: string;
}
declare enum TunnelProtocol {
    Http = "http",
    Https = "https"
}
declare enum TunnelPrivacyId {
    ConstantPrivate = "constantPrivate",// private, and changing is unsupported
    Private = "private",
    Public = "public"
}
interface TunnelCreationOptions {
    elevationRequired?: boolean;
}
interface TunnelProviderFeatures {
    elevation: boolean;
    /**
     * @deprecated
     */
    public?: boolean;
    privacyOptions: TunnelPrivacy[];
    protocol: boolean;
}
interface ITunnelProvider {
    forwardPort(tunnelOptions: TunnelOptions, tunnelCreationOptions: TunnelCreationOptions): Promise<RemoteTunnel | string | undefined> | undefined;
}
declare enum ProvidedOnAutoForward {
    Notify = 1,
    OpenBrowser = 2,
    OpenPreview = 3,
    Silent = 4,
    Ignore = 5,
    OpenBrowserOnce = 6
}
interface ProvidedPortAttributes {
    port: number;
    autoForwardAction: ProvidedOnAutoForward;
}
interface PortAttributesProvider {
    providePortAttributes(ports: number[], pid: number | undefined, commandLine: string | undefined, token: CancellationToken): Promise<ProvidedPortAttributes[]>;
}

export { type ITunnelProvider, type PortAttributesProvider, ProvidedOnAutoForward, type ProvidedPortAttributes, type RemoteTunnel, type TunnelCreationOptions, type TunnelOptions, TunnelPrivacyId, TunnelProtocol, type TunnelProviderFeatures };
